#include "Journal.h"

void outputMark(const Mark& curr)
{
	std::cout << curr.subject << ": " << curr.mark << std::endl;
}

void inputMark(Mark* curr, size_t p_size)
{
	for (int i = 0; i < p_size; i++)
	{
		std::cout << "Input subject: " << std::endl;
		std::cin >> curr[i].subject;
		std::cout << "Input mark: " << std::endl;
		std::cin >> curr[i].mark;
	}
}


void insertNewRecord(Record& newRec)
{
	std::cout << "Input course number: " << std::endl;
	std::cin >> newRec.course;
	std::cout << "Input group: " << std::endl;
	std::cin >> newRec.group;
	std::cout << "Input surname: " << std::endl;
	std::cin >> newRec.surname;
	std::cout << "Input id: " << std::endl;
	std::cin >> newRec.id;

	int count = 0;
	char input = 'q';
	while (std::cout << "Input 'y' if you want to add subject and mark or 'n' to exit: " << std::endl, std::cin >> input, input != 'n')
	{

		std::cout << "Input subject: " << std::endl;
		std::cin >> newRec.mark[count].subject;
		std::cout << "Input mark: " << std::endl;
		std::cin >> newRec.mark[count].mark;
		count++;
	}
	newRec.mark_size = count;
}

void outputRecord(const Record& rec)
{
	std::cout << "Course number: " << rec.course << std::endl;
	std::cout << "Group: " << rec.group << std::endl;
	std::cout << "Surname: " << rec.surname << std::endl;
	std::cout << "Id: " << rec.id << std::endl;
	for (int i = 0; i < rec.mark_size; i++)
	{
		outputMark(rec.mark[i]);
	}
	std::cout << std::endl;
}



void createAndWrite(const std::string& fileName) {

	char input = 'q';
	while (input != 'n') {
		std::cout << "Create a new file? Press 'y' if yes and 'n' if no." << std::endl;
		std::cin >> input;
		if (input == 'n') {
			break;
		}
		else if (input == 'y') {
			std::cout << "Creating file..." << std::endl << std::endl;
			std::ofstream newFile(fileName + ".bin", std::ios::binary | std::ios::out);

			if (!newFile.is_open()) {
				std::cout << "Something went wrong. Please try again." << std::endl << std::endl;
			}
			else {
				std::cout << "Success!" << std::endl;
				char innerInput = 'q';
				while (innerInput != 'n') {
					std::cout << "Add data? Press 'y' if yes and 'n' if no." << std::endl;
					std::cin >> innerInput;
					if (innerInput == 'n') {
						break;
					}
					else if (innerInput == 'y') {
						Record newRecord;
						insertNewRecord(newRecord);
						newFile.write(reinterpret_cast<char*>(&newRecord), sizeof(Record));
					}
				}
			}
			newFile.close();
		}
		else {
			std::cout << "I do not know what to do with such a command... Try again." << std::endl;
		}
	}
}


cl_Journal::cl_Journal(const std::string& filename) :
	base_fileName(filename)
{
	load_in();
}

cl_Journal::~cl_Journal()
{
	load_out();
}

void cl_Journal::load_in()
{
	session.clear();
	f_amount = 0;
	std::ifstream in(base_fileName + ".bin", std::ios::binary);
	if (!in.is_open())
		return;

	Record curr;
	while (in.read(reinterpret_cast<char*>(&curr), sizeof(Record)))
	{
		f_amount++;
		session.insert({ f_amount, curr });
	}

	in.close();

}

void cl_Journal::load_out()
{
	std::ofstream out(base_fileName + ".bin", std::ios::binary);

	std::map<size_t, Record>::iterator mapIter = session.begin();

	while (mapIter != session.end())
	{
		out.write(reinterpret_cast<char*>(&mapIter->second), sizeof(Record));
		mapIter++;
	}

	out.close();
}


bool cl_Journal::add_record(const Record& rec)
{
	f_amount++;
	std::pair<size_t, Record> c_p(f_amount, rec);
	session.insert(c_p);
	return true;

}

bool cl_Journal::add_record(Record&& rec)
{
	f_amount++;
	session.emplace(std::make_pair(f_amount, rec));
	return true;

}

bool cl_Journal::search_record(const Record& rec)
{
	auto res = std::find_if(session.begin(), session.end(), [&rec](const std::pair<size_t, Record>& toFind)
		{
			return toFind.second == rec;
		});
	if (res == session.end())
	{
		return false;
	}

	std::ofstream res_out(base_fileName + "LastOperation.bin", std::ios::binary | std::ios::trunc);

	while (res != session.end())
	{
		res_out.write(reinterpret_cast<char*>(&res->second), sizeof(Record));
		outputRecord(res->second);
		res++;
		res = std::find_if(res, session.end(), [&rec](const std::pair<size_t, Record>& toFind)
			{
				return toFind.second == rec;
			});
	}

	res_out.close();

	return true;

}

bool cl_Journal::search_record(const int& p_course)
{
	auto res = std::find_if(session.begin(), session.end(), [&p_course](const std::pair<size_t, Record>& toFind)
		{
			return toFind.second.course == p_course;
		});
	if (res == session.end())
	{
		return false;
	}

	std::ofstream res_out(base_fileName + "LastOperation.bin", std::ios::binary | std::ios::trunc);

	while (res != session.end())
	{
		res_out.write(reinterpret_cast<char*>(&res->second), sizeof(Record));
		outputRecord(res->second);
		res++;
		res = std::find_if(res, session.end(), [&p_course](const std::pair<size_t, Record>& toFind)
			{
				return toFind.second.course == p_course;
			});
	}

	res_out.close();

	return true;

}

bool cl_Journal::search_record_group(const std::string& p_group)
{
	auto res = std::find_if(session.begin(), session.end(), [&p_group](const std::pair<size_t, Record>& toFind)
		{
			return toFind.second.group == p_group;
		});
	if (res == session.end())
	{
		return false;
	}

	std::ofstream res_out(base_fileName + "LastOperation.bin", std::ios::binary | std::ios::trunc);

	while (res != session.end())
	{
		res_out.write(reinterpret_cast<char*>(&res->second), sizeof(Record));
		outputRecord(res->second);
		res++;
		res = std::find_if(res, session.end(), [&p_group](const std::pair<size_t, Record>& toFind)
			{
				return toFind.second.group == p_group;
			});
	}

	res_out.close();

	return true;

}

bool cl_Journal::search_record(const Mark* p_mark)
{
	auto res = std::find_if(session.begin(), session.end(), [&p_mark](const std::pair<size_t, Record>& toFind)
		{
			bool tmp = false;
			for (int i = 0; i < toFind.second.mark_size; i++)
			{
				if (!(strcmp(toFind.second.mark[i].subject, p_mark[i].subject) == 0 && toFind.second.mark[i].mark == p_mark[i].mark))
				{
					tmp = false;
					break;
				}
				else tmp = true;
			}
			return tmp;
		});
	if (res == session.end())
	{
		return false;
	}

	std::ofstream res_out(base_fileName + "LastOperation.bin", std::ios::binary | std::ios::trunc);

	while (res != session.end())
	{
		res_out.write(reinterpret_cast<char*>(&res->second), sizeof(Record));
		outputRecord(res->second);
		res++;
		res = std::find_if(res, session.end(), [&p_mark](const std::pair<size_t, Record>& toFind)
			{
				bool tmp = false;
				for (int i = 0; i < toFind.second.mark_size; i++)
				{
					if (!(strcmp(toFind.second.mark[i].subject, p_mark[i].subject) == 0 && toFind.second.mark[i].mark == p_mark[i].mark))
					{
						tmp = false;
						break;
					}
					else tmp = true;
				}
				return tmp;
			});
	}

	res_out.close();

	return true;
}



bool cl_Journal::search_record_surname(const std::string& p_surname)
{
	auto res = std::find_if(session.begin(), session.end(), [&p_surname](const std::pair<size_t, Record>& toFind)
		{
			return toFind.second.surname == p_surname;
		});
	if (res == session.end())
	{
		return false;
	}

	std::ofstream res_out(base_fileName + "LastOperation.bin", std::ios::binary | std::ios::trunc);

	while (res != session.end())
	{
		res_out.write(reinterpret_cast<char*>(&res->second), sizeof(Record));
		outputRecord(res->second);
		res++;
		res = std::find_if(res, session.end(), [&p_surname](const std::pair<size_t, Record>& toFind)
			{
				return toFind.second.surname == p_surname;
			});
	}

	res_out.close();

	return true;
}

bool cl_Journal::search_record_id(const std::string& p_id)
{
	auto res = std::find_if(session.begin(), session.end(), [&p_id](const std::pair<size_t, Record>& toFind)
		{
			return toFind.second.id == p_id;
		});
	if (res == session.end())
	{
		return false;
	}

	std::ofstream res_out(base_fileName + "LastOperation.bin", std::ios::binary | std::ios::trunc);

	while (res != session.end())
	{
		res_out.write(reinterpret_cast<char*>(&res->second), sizeof(Record));
		outputRecord(res->second);
		res++;
		res = std::find_if(res, session.end(), [&p_id](const std::pair<size_t, Record>& toFind)
			{
				return toFind.second.id == p_id;
			});
	}

	res_out.close();

	return true;
}


bool cl_Journal::delete_record(const Record& rec)
{
	auto res = std::find_if(session.begin(), session.end(), [&rec](const std::pair<size_t, Record>& toFind)
		{
			return toFind.second == rec;
		});
	if (res == session.end())
	{
		return false;
	}

	while (res != session.end())
	{
		res = session.erase(res);
		f_amount--;
		res = std::find_if(res, session.end(), [&rec](const std::pair<size_t, Record>& toFind)
			{
				return toFind.second == rec;
			});
	}
	return true;

}

bool cl_Journal::change_record(const Record& toChange, const Record& toImply)
{
	auto res = std::find_if(session.begin(), session.end(), [&toChange](const std::pair<size_t, Record>& toFind)
		{
			return toFind.second == toChange;
		});
	if (res == session.end())
	{
		return false;
	}

	while (res != session.end())
	{
		res->second = toImply;
		res++;
		res = std::find_if(session.begin(), session.end(), [&toChange](const std::pair<size_t, Record>& toFind)
			{
				return toFind.second == toChange;
			});
	}

	return true;

}

bool cl_Journal::avg_subject(const std::string& p_subject)
{
	double avg_mark = 0;
	int count = 0;
	std::for_each(session.begin(), session.end(), [&p_subject, &avg_mark, &count](const std::pair<size_t, Record>& toFind)
		{
			for (int i = 0; i < toFind.second.mark_size; i++)
			{
				if (toFind.second.mark[i].subject == p_subject)
				{
					avg_mark += toFind.second.mark[i].mark;
					count++;
				}
			}
		});
	avg_mark /= count;
	std::cout << "Average mark for " << p_subject << " is " << avg_mark << std::endl;
	return true;
}

bool cl_Journal::avg_course(int p_course)
{
	double avg_mark = 0;
	int count = 0;
	std::for_each(session.begin(), session.end(), [&p_course, &avg_mark, &count](const std::pair<size_t, Record>& toFind)
		{
			if (toFind.second.course == p_course)
			{
				for (int i = 0; i < toFind.second.mark_size; i++)
				{
					avg_mark += toFind.second.mark[i].mark;
					count++;
				}
			}
		});
	avg_mark /= count;
	std::cout << "Average mark for " << p_course << " course is " << avg_mark << std::endl;
	return true;
}

bool cl_Journal::avg_group(const std::string& p_group)
{
	double avg_mark = 0;
	int count = 0;
	std::for_each(session.begin(), session.end(), [&p_group, &avg_mark, &count](const std::pair<size_t, Record>& toFind)
		{
			if (toFind.second.group == p_group)
			{
				for (int i = 0; i < toFind.second.mark_size; i++)
				{
					avg_mark += toFind.second.mark[i].mark;
					count++;
				}
			}
		});
	avg_mark /= count;
	std::cout << "Average mark for group " << p_group << " is " << avg_mark << std::endl;
	return true;
}

void cl_Journal::loadOutTXT()
{
	std::ofstream txt_out(base_fileName + ".txt");
	txt_out << "Course     " << "Group     " << "Surname     " << "ID     " << "Subjects     " << std::endl;
	txt_out << "____________________________________________________" << std::endl;
	auto Iter = session.begin();
	while (Iter != session.end())
	{
		txt_out << "   " << Iter->second.course << "     " << Iter->second.group << "     " << Iter->second.surname << "     " << Iter->second.id << "     ";
		for (int i = 0; i < Iter->second.mark_size; i++)
		{
			txt_out << Iter->second.mark[i] << "  ";
		}
		Iter++;
		txt_out << std::endl;
	}

	txt_out.close();
}
